import { defineStore } from 'pinia';
import { ref } from 'vue';
import type { OrderPriority } from '@/types/OrderPriority';
import * as service from '@/services/orderPriority';

export const useOrderPriorityStore = defineStore('orderPriority', () => {
  const priorities = ref<OrderPriority[]>([]);
  const loading = ref(false);
  const error = ref<string | null>(null);

  async function fetchAll() {
    loading.value = true;
    error.value = null;
    try {
      priorities.value = await service.listOrderPriorities();
    } catch (e: any) {
      error.value = e.message || 'Unknown Error';
    } finally {
      loading.value = false;
    }
  }

  async function fetchWithFilter(orderId?: number, pageId?: number) {
    loading.value = true;
    error.value = null;
  
    // ✅ แปลงและกรองค่าที่ไม่ใช่ number
    const validOrderId = typeof orderId === 'number' && !isNaN(orderId) ? orderId : undefined;
    const validPageId = typeof pageId === 'number' && !isNaN(pageId) ? pageId : undefined;
  
    try {
      priorities.value = await service.findOrderPrioritiesWithFilter(validOrderId, validPageId);
      console.log
    } catch (e: any) {
      error.value = e.message || 'Unknown Error';
    } finally {
      loading.value = false;
    }
  }
  
  

  async function createOrUpdate(payload: service.CreateOrderPriorityPayload) {
    loading.value = true;
    error.value = null;
    try {
      const result = await service.createOrUpdateOrderPriority(payload);
      await fetchAll(); // หรืออัปเดต state แบบเฉพาะจุดก็ได้
      return result;
    } catch (e: any) {
      error.value = e.message || 'Unknown Error';
    } finally {
      loading.value = false;
    }
  }

  async function updateMany(payload: service.BatchUpdateManyOrderPriorityDto) {
    loading.value = true;
    error.value = null;
    try {
      await service.updateManyOrderPriorities(payload);
      await fetchAll();
    } catch (e: any) {
      error.value = e.message || 'Unknown Error';
    } finally {
      loading.value = false;
    }
  }

  async function remove(id: number) {
    loading.value = true;
    error.value = null;
    try {
      await service.deleteOrderPriority(id);
      priorities.value = priorities.value.filter(p => p.orderPriorityID !== id);
    } catch (e: any) {
      error.value = e.message || 'Unknown Error';
    } finally {
      loading.value = false;
    }
  }

  return {
    priorities,
    loading,
    error,
    fetchAll,
    fetchWithFilter,
    createOrUpdate,
    updateMany,
    remove,
  };
});
