import { defineStore } from 'pinia'
import { ref } from 'vue'
import type { Queue, QueuePostData } from '@/types/Queue'
import * as queueService from '@/services/queue'
import { runScheduleDirect } from '@/services/queue'
import { refreshAllData } from './refresh/refreshAll'
import { useStockConfigStore } from './stockConfig'
import { useDateStore } from './dateStore'
import { usePageContextStore } from './pageContext'

export const useQueueStore = defineStore('queue', () => {
  const queues = ref<Queue[]>([])
  const loading = ref<boolean>(false)
  const error = ref<string | null>(null)
  const stockConfigStore = useStockConfigStore()
  const dateStore = useDateStore()
  const pageContextStore = usePageContextStore()
  // **ดึงรายการ Queue ทั้งหมดจาก backend**
  async function fetchQueues() {
    loading.value = true
    error.value = null
    try {
      console.log('📡 Fetching Queues from Backend...')
      const response = await queueService.listQueues()
      console.log('✅ Queues Response from Backend:', response)
      queues.value = response
    } catch (e: any) {
      console.error('❌ Error fetching queues:', e.response?.data || e.message)
      error.value = e.message || 'Unknown Error'
    } finally {
      loading.value = false
    }
  }

  // **ดึง Queue รายการเดียวจาก backend**
  async function getQueue(id: number): Promise<Queue> {
    loading.value = true
    error.value = null
    try {
      console.log(`📡 Fetching Queue ID: ${id} from Backend...`)
      const queue = await queueService.getQueue(id)
      console.log('✅ Queue Data Received:', queue)
      return queue
    } catch (e: any) {
      console.error(`❌ Error fetching Queue ID ${id}:`, e.response?.data || e.message)
      error.value = e.message || 'Unknown Error'
      throw e
    } finally {
      loading.value = false
    }
  }

  // **เพิ่ม Queue ลง Database (ใช้ QueuePostData)**
  async function addQueue(newQueue: QueuePostData) {
    loading.value = true
    error.value = null
    try {
      console.log('➕ Adding Queue:', newQueue)
      const createdQueue = await queueService.createQueue(newQueue)
      queues.value.push(createdQueue)
      console.log('✅ Queue Added:', createdQueue)
    } catch (e: any) {
      console.error('❌ Error adding queue:', e.response?.data || e.message)
      error.value = e.message || 'Unknown Error'
    } finally {
      loading.value = false
    }
  }

  // **อัปเดต Queue (ใช้ QueuePostData)**
  async function updateQueue(id: number, updatedData: QueuePostData) {
    if (!id) {
      console.error('❌ queueID is undefined! Cannot update.')
      return
    }

    loading.value = true
    error.value = null
    try {
      console.log(`🛠 Updating Queue ID: ${id}...`)
      const updatedQueue = await queueService.updateQueue(id, updatedData)
      const index = queues.value.findIndex((q) => q.QueueID === id)
      if (index !== -1) {
        queues.value[index] = updatedQueue
        console.log(`✅ Queue ID: ${id} Updated Successfully`, updatedQueue)
      } else {
        console.warn(`⚠️ Queue ID: ${id} Not Found in Store!`)
      }
    } catch (e: any) {
      console.error(`❌ Error updating Queue ID ${id}:`, e.response?.data || e.message)
      error.value = e.message || 'Unknown Error'
    } finally {
      loading.value = false
    }
  }

  // **ลบ Queue**
  async function deleteQueue(id: number) {
    loading.value = true
    error.value = null
    try {
      console.log(`🗑 Deleting Queue ID: ${id}...`)
      await queueService.deleteQueue(id)
      // ลบ Queue ออกจาก Store
      queues.value = queues.value.filter((q) => q.QueueID !== id)
      console.log(`✅ Queue ID: ${id} Deleted Successfully`)
    } catch (e: any) {
      console.error(`❌ Error deleting Queue ID ${id}:`, e.response?.data || e.message)
      error.value = e.message || 'Unknown Error'
    } finally {
      loading.value = false
    }
  }

  function convertQueueToPostData(queue: Queue): QueuePostData {
    return {
      startTime: new Date(queue.startTime).toISOString(),
      finishTime: new Date(queue.finishTime).toISOString(),
      status: queue.status,
      bottleSize: queue.bottleSize,
      producedQuantity: queue.producedQuantity,
      MachineID: queue.machine?.MachineID ?? 0, // ใช้ MachineID จาก object
      PageID: queue.page?.PageID ?? 0,
      OrderID: queue.order?.OrderID ?? 0,
      EmployeeIds: queue.employees.map((emp) => emp.EmployeeID) // แปลง employees เป็นแค่ array ของ ID
    }
  }

  const triggerRunSchedule = async () => {
    const payload = {
      produceBuffer: stockConfigStore.isProductionSpare, // ✅ ผูกกับ v-switch
      secondRound: false,
      startDate: dateStore.currentDate,
      pageID: pageContextStore.currentPage
    }

    console.log('📦 Payload ที่จะส่งไป:', payload)

    try {
      const result = await runScheduleDirect(payload)
      console.log('✅ Schedule run success:', result)

      await refreshAllData()
    } catch (err) {
      console.error('❌ Schedule run failed:', err)
    }
  }

  return {
    queues,
    loading,
    error,
    fetchQueues,
    getQueue,
    convertQueueToPostData,
    addQueue,
    updateQueue,
    deleteQueue,
    triggerRunSchedule
  }
})
